@@ -1,15 +1,24 @@
+"""
+Trust Region Steps
+------------------
+This module provides the machinery to compute different trust-region(
+-reflective) step proposals and select among them based on to their
+performance according to the quadratic approximation of the objective function
+"""
+
 import numpy as np
-from .logging import logger
+import scipy.linalg as linalg
 
 from numpy.linalg import norm
 from scipy.sparse import csc_matrix
-import scipy.linalg as linalg
 
 from typing import Tuple
 
+from .logging import logger
 from .subproblem import (
     solve_1d_trust_region_subproblem, solve_nd_trust_region_subproblem
 )
+from .constants import SubSpaceDim
 
 
 def normalize(v: np.ndarray) -> None:
@@ -65,6 +74,29 @@ class Step:
                  theta: float,
                  ub: np.ndarray,
                  lb: np.ndarray):
+        """
+
+        :param x:
+            Reference point
+        :param sg:
+            Gradient in rescaled coordinates
+        :param hess:
+            Hessian in unscaled coordinates
+        :param scaling:
+            Matrix that defines scaling transformation
+        :param g_dscaling:
+            Unscaled gradient multiplied by derivative of scaling
+            transformation
+        :param delta:
+            Trust region Radius in scaled coordinates
+        :param theta:
+            Stepback parameter that controls how close steps are allowed to
+            get to the boundary
+        :param ub:
+            Upper boundary
+        :param lb:
+            Lower boundary
+        """
         self.x = x
 
         self.s = None
@@ -192,6 +224,10 @@ class TRStep2D(Step):
 
     def __init__(self, x, sg, hess, scaling, g_dscaling, delta, theta,
                  ub, lb, subspace):
+        """
+        :param subspace:
+            Precomputed subspace
+        """
         super().__init__(x, sg, hess, scaling, g_dscaling, delta, theta,
                          ub, lb)
         self.subspace = subspace
@@ -199,17 +235,17 @@ class TRStep2D(Step):
             n = len(sg)
 
             s_newt = linalg.solve(hess, sg)
-            posdef = s_newt.dot(hess.dot(s_newt)) <= 0
+            posdef = s_newt.dot(hess.dot(s_newt)) > 0
             normalize(s_newt)
 
             if n > 1:
-                if posdef:
+                if not posdef:
                     # in this case we are in Case 2 of Fig 12 in
                     # [Coleman-Li1994]
                     logger.debug('Newton direction did not have negative '
                                  'curvature adding scaling * np.sign(sg) to '
                                  '2D subspace.')
-                    s_grad = scaling * np.sign(sg)
+                    s_grad = scaling * np.sign(sg) + (sg == 0)
                 else:
                     s_grad = sg.copy()
 
@@ -218,7 +254,7 @@ class TRStep2D(Step):
                 s_grad = s_grad - s_newt * (s_newt.dot(s_grad))
                 normalize(s_grad)
                 # if non-zero, add s_grad to subspace
-                if np.any(s_grad):
+                if np.any(s_grad != 0):
                     self.subspace = np.vstack([s_newt, s_grad]).T
                     return
                 else:
@@ -238,6 +274,10 @@ class TRStepReflected(Step):
 
     def __init__(self, x, sg, hess, scaling, g_dscaling, delta, theta,
                  ub, lb, tr_step):
+        """
+        :param tr_step:
+            Trust-region step that is reflected
+        """
         super().__init__(x, sg, hess, scaling, g_dscaling, delta, theta,
                          ub, lb)
 
@@ -280,11 +320,12 @@ def trust_region_reflective(x: np.ndarray,
                             dv: np.ndarray,
                             theta: float,
                             lb: np.ndarray,
-                            ub: np.ndarray) -> Tuple[np.ndarray,
-                                                     np.ndarray,
-                                                     float,
-                                                     np.ndarray,
-                                                     str]:
+                            ub: np.ndarray,
+                            subspace_dim: SubSpaceDim) -> Tuple[np.ndarray,
+                                                                np.ndarray,
+                                                                float,
+                                                                np.ndarray,
+                                                                str]:
     """
     Compute a step according to the solution of the trust-region subproblem.
     If step-back is necessary, gradient and reflected trust region step are
@@ -313,26 +354,32 @@ def trust_region_reflective(x: np.ndarray,
         lower optimization variable boundaries
     :param ub:
         upper optimization variable boundaries
+    :param subspace_dim:
+        Subspace dimension in which the subproblem will be solved. Larger
+        subspaces require more compute time but can yield higher quality step
+        proposals.
 
     :return:
-        s: proposed step
-        ss: rescaled proposed step
+        s: proposed step,
+        ss: rescaled proposed step,
         qpval: expected function value according to local quadratic
-        approximation
-        subspace: computed subspace for reuse if proposed step is not accepted
+        approximation,
+        subspace: computed subspace for reuse if proposed step is not accepted,
         steptype: type of step that was selected for proposal
     """
     sg = scaling.dot(g)
     g_dscaling = csc_matrix(np.diag(np.abs(g) * dv))
 
-    if hess.shape[0] > 100:
+    if subspace_dim == SubSpaceDim.TWO:
         tr_step = TRStep2D(
             x, sg, hess, scaling, g_dscaling, delta, theta, ub, lb, tr_subspace
         )
-    else:
+    elif subspace_dim == SubSpaceDim.FULL:
         tr_step = TRStepFull(
             x, sg, hess, scaling, g_dscaling, delta, theta, ub, lb,
         )
+    else:
+        raise ValueError('Invalid choice of subspace dimension.')
     tr_step.calculate()
 
     # in case of truncation, we hit the boundary and we check both the
